#include "SamplePlugin.h"
#include "Sample_SSAO.h"

using namespace Ogre;
using namespace OgreBites;

#define SSAO_GUI_WIDTH 250
#define SSAO_GUI_TRACK_WIDTH 100
#define SSAO_GUI_VALUE_BOX_WIDTH 50
#define SSAO_PI 3.1415926535897932384626433832795028841971693993751

const String SSAO_THUMBNAIL = "thumb_ssao.png";

const String SSAO_SCREENSHOT_NAME = "screenshot";
const String SSAO_OBJECT_MENU_NAME = "ObjectType";
const String SSAO_CAMERA_MENU_NAME = "Camera";

const String SSAO_COMPOSITOR_MENU_NAME = "compositor";
const String SSAO_POST_MENU_NAME = "post";
const String SSAO_CREASE_MINIMUM_NAME = "CreaseMinimum";
const String SSAO_CREASE_RANGE_NAME = "mCreaseRange";
const String SSAO_CREASE_BIAS_NAME = "mCreaseBias";
const String SSAO_CREASE_AVERAGER_NAME = "mCreaseAverager";
const String SSAO_CREASE_KERNELSIZE_NAME = "mCreaseKernelsize";

const String SSAO_SAMPLE_SPACE_NAME = "sampleSpace";
const String SSAO_SAMPLE_LENGTH_SCREENSPACE = "sampleScreenSpace";
const String SSAO_SAMPLE_LENGTH_WORLDSPACE = "sampleWorldSpace";
const String SSAO_SAMPLE_LENGTH_EXPONENT_NAME = "sampleLengthExponent";

const String SSAO_ANGLE_BIAS_NAME = "angleBias";

const String SSAO_CRYTEK_OFFSET_SCALE_NAME = "offsetScale";
const String SSAO_CRYTEK_EDGE_HIGHLIGHT_NAME = "edgeHighlight";
const String SSAO_CRYTEK_DEFAULT_ACCESSIBILITY_NAME = "defaultOcclusion";

const String SSAO_UNSHARP_KERNEL_BIAS_NAME = "kernelBias";
const String SSAO_UNSHARP_LAMBDA_NAME = "lambda";

const String SSAO_BILATERAL_PHOTOMETRIC_EXPONENT = "photometricExponent";

const String SSAO_USER_CAMERA_ITEM = "User Camera";
const String SSAO_CAMERA_SIBENIK = "Sibenik";
const String SSAO_CAMERA_CORNELL = "Cornell Box";

SamplePlugin* sp;
Sample* s;

extern "C" _OgreSampleExport void dllStartPlugin()
{
	s = new Sample_SSAO();
	sp = OGRE_NEW SamplePlugin(s->getInfo()["Title"] + " Sample");
	sp->addSample(s);
	Root::getSingleton().installPlugin(sp);
}

extern "C" _OgreSampleExport void dllStopPlugin()
{
	Root::getSingleton().uninstallPlugin(sp);
	OGRE_DELETE sp;
	delete s;
}

Sample_SSAO::Sample_SSAO()
{
	mInfo["Title"] = "SSAO techniques";
	mInfo["Description"] = "A demo showing a few ssao shading techniques.";
	mInfo["Thumbnail"] = SSAO_THUMBNAIL;
	mInfo["Category"] = "Unsorted";

	mMeshNames.push_back("sibenik");
	mMeshNames.push_back("cornell");
	
	mCompositorNames.push_back("SSAO/HemisphereMC");
	mCompositorNames.push_back("SSAO/Volumetric");
	mCompositorNames.push_back("SSAO/HorizonBased");
	mCompositorNames.push_back("SSAO/Crytek");
	mCompositorNames.push_back("SSAO/CreaseShading");
	mCompositorNames.push_back("SSAO/UnsharpMask");
	mCompositorNames.push_back("SSAO/ShowDepth");
	mCompositorNames.push_back("SSAO/ShowNormals");
	mCompositorNames.push_back("SSAO/ShowViewPos");

	mPostNames.push_back("SSAO/Post/NoFilter");
	mPostNames.push_back("SSAO/Post/CrossBilateralFilter");
	mPostNames.push_back("SSAO/Post/SmartBoxFilter");
	mPostNames.push_back("SSAO/Post/BoxFilter");
		
	mCurrentCompositor = mCompositorNames[0];
	mCurrentPost = mPostNames[0];

	mTakeScreenshot = false;
}

StringVector Sample_SSAO::getRequiredPlugins()
{
	StringVector names;
	names.push_back("Cg Program Manager");
	return names;
}

void Sample_SSAO::testCapabilities(const RenderSystemCapabilities* caps)
{
	if (!caps->hasCapability(RSC_VERTEX_PROGRAM) || !caps->hasCapability(RSC_FRAGMENT_PROGRAM))
    {
		OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED, "Your graphics card does not support vertex and fragment"
			" programs, so you cannot run this sample. Sorry!", "Sample_SSAO::testCapabilities");
    }
}

void Sample_SSAO::setupControls(void)
{
	mTrayMgr->createButton(TL_TOPLEFT, SSAO_SCREENSHOT_NAME, "take screenshot w/o ui", SSAO_GUI_WIDTH);

	// --- select mesh menu ---
	SelectMenu* objectType = mTrayMgr->createThickSelectMenu(TL_TOPLEFT, SSAO_OBJECT_MENU_NAME, "Object: ", SSAO_GUI_WIDTH, 16);
	for (unsigned int i = 0; i < mMeshNames.size(); i++)
		objectType->addItem(mMeshNames[i]);

	// --- select camera menu ---
	mCameraMenu = mTrayMgr->createThickSelectMenu(TL_TOPLEFT, SSAO_CAMERA_MENU_NAME, "Camera position", SSAO_GUI_WIDTH, 16);
	mCameraMenu->addItem(SSAO_USER_CAMERA_ITEM);
	mCameraMenu->addItem(SSAO_CAMERA_CORNELL);
	mCameraMenu->addItem(SSAO_CAMERA_SIBENIK);

	// --- select compositor menu ---
	SelectMenu* compositor = mTrayMgr->createThickSelectMenu(TL_TOPLEFT, SSAO_COMPOSITOR_MENU_NAME, "Compositor: ", SSAO_GUI_WIDTH, 16);
	for (unsigned int i = 0; i < mCompositorNames.size(); i++)
		compositor->addItem(mCompositorNames[i]);

	// --- select post filter menu ---
	SelectMenu * post = mTrayMgr->createThickSelectMenu(TL_TOPLEFT, SSAO_POST_MENU_NAME, "Post filter: ", SSAO_GUI_WIDTH, 16);
	for (unsigned int i = 0; i < mPostNames.size(); i++)
		post->addItem(mPostNames[i]);

	// --- hemisphere MC sample length exponent --- //
	mSampleLengthExponent = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_SAMPLE_LENGTH_EXPONENT_NAME,
		"sample length exponent",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		5,
		501); // snaps ???
	

	// --- bilateral photometric exponent ---
	Slider* photometricExponent = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_BILATERAL_PHOTOMETRIC_EXPONENT,
		"photometric exponent",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		50,
		501); // snaps ???
	mBilateralPanel.push_back(photometricExponent);
	//mBilateralPanel.push_back(mTrayMgr->createSeparator(TL_TOPLEFT, "sepExponent", SSAO_GUI_WIDTH));

	// --- crease shading options ---
	Slider* mCreaseMinimumSlider = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_CREASE_MINIMUM_NAME,
		"Minimum Crease",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		1,
		101); // snaps ???
	mCreasePanel.push_back(mCreaseMinimumSlider);

	Slider* mCreaseRange = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_CREASE_RANGE_NAME,
		"sqrt(Range)",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		10,
		101); // snaps ???
	mCreasePanel.push_back(mCreaseRange);

	Slider* mCreaseBias = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_CREASE_BIAS_NAME,
		"Bias",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		2,
		101); // snaps ???
	mCreasePanel.push_back(mCreaseBias);

	Slider* mCreaseAverager = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_CREASE_AVERAGER_NAME,
		"Averager",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		100,
		101); // snaps ???
	mCreasePanel.push_back(mCreaseAverager);

	Slider* mCreaseKernelsize = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_CREASE_KERNELSIZE_NAME,
		"Kernel size bias",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		10,
		101); // snaps ???
	mCreasePanel.push_back(mCreaseKernelsize);

	// --- sample length parameter ---
	mTrayMgr->createSeparator(TL_TOPLEFT, "sep");
	mSamplingCheckbox = mTrayMgr->createCheckBox(TL_TOPLEFT, SSAO_SAMPLE_SPACE_NAME, "sample in screen space", SSAO_GUI_WIDTH);
	mScreenSpaceSlider = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_SAMPLE_LENGTH_SCREENSPACE,
		"screen space length (in %)",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		100,
		10001);

	mWorldSpaceSlider = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_SAMPLE_LENGTH_WORLDSPACE,
		"world space length (units)",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		10,
		10001);

	// --- angle bias ---
	mAngleBiasSlider = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_ANGLE_BIAS_NAME,
		"angle Bias (radians)",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		SSAO_PI/2,
		1001);

	// --- offset length ---
	mOffsetStepSlider = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_CRYTEK_OFFSET_SCALE_NAME,
		"offset Scale (% of sample length)",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		100,
		10001);

	// --- crytek edge highlight ---
	mEdgeHighlight = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_CRYTEK_EDGE_HIGHLIGHT_NAME,
		"edge highlight factor",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		1,
		101);

	// --- crytek default accesibility value for invalid samples ---
	mDefaultAccessibility = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_CRYTEK_DEFAULT_ACCESSIBILITY_NAME,
		"default accesibility",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		1,
		101);

	// --- unsharp mask kernel bias ---
	Slider* mUnsharpKernelBias = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_UNSHARP_KERNEL_BIAS_NAME,
		"Kernel size bias",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		10,
		101); // snaps ???
	mUnsharpPanel.push_back(mUnsharpKernelBias);

	// --- unsharp mask lambda ---
	Slider* mUnsharpLambda = mTrayMgr->createThickSlider(TL_TOPLEFT,
		SSAO_UNSHARP_LAMBDA_NAME,
		"sqrt(lambda)",
		SSAO_GUI_WIDTH,
		SSAO_GUI_VALUE_BOX_WIDTH,
		0,
		10,
		101); // snaps ???
	mUnsharpPanel.push_back(mUnsharpLambda);

	// setup values
	mCreaseMinimumSlider->setValue(0.2);
	mCreaseRange->setValue(1.0f);
	mCreaseBias->setValue(1.0f);
	mCreaseAverager->setValue(24);
	mCreaseKernelsize->setValue(3.0f);

	mSamplingCheckbox->setChecked(false);
	mScreenSpaceSlider->setValue(6.0f);
	mWorldSpaceSlider->setValue(2.0f);
	mAngleBiasSlider->setValue(0.2f);
	mOffsetStepSlider->setValue(1.0f);
	mEdgeHighlight->setValue(0.0f);
	mDefaultAccessibility->setValue(0.5f);

	mUnsharpKernelBias->setValue(1.0f);
	mUnsharpLambda->setValue(5.0f);

	photometricExponent->setValue(10.0f);

	mSampleLengthExponent->setValue(1.0f);


	mTrayMgr->showCursor();
}

void Sample_SSAO::setupContent()
{
	mViewport->setBackgroundColour(ColourValue(0.5, 0.5, 0.5, 1));

	// set our camera to orbit around the origin and show cursor
	mCameraMan->setStyle(CS_FREELOOK);
	mCamera->move(Vector3(0, 10, 0));
	mCamera->setFOVy(Radian(Degree(45).valueRadians())); // i.e. 60deg * 1.3.. maya and ogre use fovX and fovY
	mCamera->setFarClipDistance(400);
	mCamera->setNearClipDistance(0.1);
	mTrayMgr->showCursor();

	// init the camera position/orientation
	// cornell
	//mCamera->setPosition(0, 5, 20);
	//mCamera->lookAt(Vector3(0, 5, 0));
	
	// sibenik
	mCamera->setPosition(27, 9, -2);
	mCamera->lookAt(Vector3(-6, 2, 1));;

	// setup all mMeshes
	for (unsigned int i = 0; i < mMeshNames.size(); i++) {
		Entity* ent = mSceneMgr->createEntity(mMeshNames[i], mMeshNames[i] + ".mesh");
		ent->setVisible(false);
		ent->setMaterialName("SSAO/GBuffer");

		mSceneMgr->getRootSceneNode()->attachObject(ent);
		mMeshes.push_back(ent);
	}
	mCurrentMeshIndex = 0;
	mMeshes[mCurrentMeshIndex]->setVisible(true);

	setupCompositors();

	setupControls();

	changeCompositor(mCompositorNames[0]);
	changePost(mPostNames[0]);
}

void Sample_SSAO::setupCompositors()
{

	if (CompositorManager::getSingleton().addCompositor(mViewport, "SSAO/GBuffer"))
		CompositorManager::getSingleton().setCompositorEnabled(mViewport, "SSAO/GBuffer", true);
	else
		LogManager::getSingleton().logMessage("SSAO: Failed to add GBuffer compositor \n");

	for (unsigned int i = 0; i < mCompositorNames.size(); i++)
	{
		if (CompositorManager::getSingleton().addCompositor(mViewport, mCompositorNames[i]))
			CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCompositorNames[i], false);
		else
			LogManager::getSingleton().logMessage("SSAO: Failed to add compositor: " + mCompositorNames[i] + "\n");
	}

	for (unsigned int i = 0; i < mPostNames.size(); i++)
	{

		if (CompositorManager::getSingleton().addCompositor(mViewport, mPostNames[i]))
			CompositorManager::getSingleton().setCompositorEnabled(mViewport, mPostNames[i], false);
		else
			LogManager::getSingleton().logMessage("SSAO: Failed to add " + mPostNames[i] + " compositor \n");
	}
	
	CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentCompositor, true);
	CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentPost, true);
}


void Sample_SSAO::changeMesh(int index)
{
	mMeshes[mCurrentMeshIndex]->setVisible(false);
	mMeshes[index]->setVisible(true);
	mCurrentMeshIndex = index;
}

void Sample_SSAO::sliderMoved(Slider* slider)
{
	// FIXME: make switch case
	if (slider->getName() == SSAO_CREASE_MINIMUM_NAME)
		setUniform("SSAO/CreaseShading", "SSAO/CreaseShading", "cMinimumCrease", slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_CREASE_BIAS_NAME)
		setUniform("SSAO/CreaseShading", "SSAO/CreaseShading", "cBias", slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_CREASE_AVERAGER_NAME)
		setUniform("SSAO/CreaseShading", "SSAO/CreaseShading", "cAverager", slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_CREASE_RANGE_NAME)
		setUniform("SSAO/CreaseShading", "SSAO/CreaseShading", "cRange", slider->getValue() * slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_CREASE_KERNELSIZE_NAME)
		setUniform("SSAO/CreaseShading", "SSAO/CreaseShading", "cKernelSize", slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_SAMPLE_LENGTH_SCREENSPACE)
	{
		setUniform("SSAO/Crytek", "SSAO/Crytek", "cSampleLengthScreenSpace", slider->getValue()/100.0f, false, 1);
		setUniform("SSAO/HorizonBased", "SSAO/HorizonBased", "cSampleLengthScreenSpace", slider->getValue()/100.0f, false, 1);
		setUniform("SSAO/HemisphereMC", "SSAO/HemisphereMC", "cSampleLengthScreenSpace", slider->getValue()/100.0f, false, 1);
		setUniform("SSAO/Volumetric", "SSAO/Volumetric", "cSampleLengthScreenSpace", slider->getValue()/100.0f, false, 1);
	}
	else if (slider->getName() == SSAO_SAMPLE_LENGTH_WORLDSPACE)
	{
		setUniform("SSAO/Crytek", "SSAO/Crytek", "cSampleLengthWorldSpace", slider->getValue(), false, 1);
		setUniform("SSAO/HorizonBased", "SSAO/HorizonBased", "cSampleLengthWorldSpace", slider->getValue(), false, 1);
		setUniform("SSAO/HemisphereMC", "SSAO/HemisphereMC", "cSampleLengthWorldSpace", slider->getValue(), false, 1);
		setUniform("SSAO/Volumetric", "SSAO/Volumetric", "cSampleLengthWorldSpace", slider->getValue(), false, 1);
	}

	else if (slider->getName() == SSAO_ANGLE_BIAS_NAME)
		setUniform("SSAO/HorizonBased", "SSAO/HorizonBased", "cAngleBias", slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_CRYTEK_OFFSET_SCALE_NAME)
		setUniform("SSAO/Crytek", "SSAO/Crytek", "cOffsetScale", slider->getValue()/100, false, 1);

	else if (slider->getName() == SSAO_CRYTEK_EDGE_HIGHLIGHT_NAME)
		setUniform("SSAO/Crytek", "SSAO/Crytek", "cEdgeHighlight", 2.0f - slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_CRYTEK_DEFAULT_ACCESSIBILITY_NAME)
		setUniform("SSAO/Crytek", "SSAO/Crytek", "cDefaultAccessibility", slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_UNSHARP_KERNEL_BIAS_NAME)
	{
		setUniform("SSAO/UnsharpMask", "SSAO/UnsharpMask/GaussianBlurY", "cKernelWidthBias", slider->getValue(), false, 1);
		setUniform("SSAO/UnsharpMask", "SSAO/UnsharpMask/GaussianBlurX", "cKernelWidthBias", slider->getValue(), false, 1);
	}
	
	else if (slider->getName() == SSAO_UNSHARP_LAMBDA_NAME)
		setUniform("SSAO/UnsharpMask", "SSAO/UnsharpMask", "cLambda", slider->getValue() * slider->getValue(), false, 1);

	else if (slider->getName() == SSAO_BILATERAL_PHOTOMETRIC_EXPONENT)
	{
		setUniform("SSAO/Post/CrossBilateralFilter", "SSAO/HorizonBased/CrossBilateralFilter/X", "cPhotometricExponent", slider->getValue(), false);
		setUniform("SSAO/Post/CrossBilateralFilter", "SSAO/HorizonBased/CrossBilateralFilter/Y", "cPhotometricExponent", slider->getValue(), false);
	}

	else if(slider->getName() == SSAO_SAMPLE_LENGTH_EXPONENT_NAME)
		setUniform("SSAO/HemisphereMC", "SSAO/HemisphereMC", "cSampleLengthExponent", slider->getValue(), false, 1);



	CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentCompositor, true);
	CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentPost, true);
}

void Sample_SSAO::itemSelected(SelectMenu* menu)
{
	if (menu->getName() == SSAO_OBJECT_MENU_NAME)
		changeMesh(menu->getSelectionIndex());

	else if (menu->getName() == SSAO_COMPOSITOR_MENU_NAME)
		changeCompositor(menu->getSelectedItem());
	
	else if (menu->getName() == SSAO_POST_MENU_NAME)
		changePost(menu->getSelectedItem());	
	
	else if (menu->getName() == SSAO_CAMERA_MENU_NAME)
	{
		if (menu->getSelectedItem() == SSAO_CAMERA_SIBENIK)
		{
			mCamera->setPosition(27, 9, -2);
			mCamera->lookAt(Vector3(-6, 2, 1));
		}
		else if (menu->getSelectedItem() == SSAO_CAMERA_CORNELL)
		{
			mCamera->setPosition(0, 5, 20);
			mCamera->lookAt(Vector3(0, 5, 0));
		}
	}
}


void Sample_SSAO::changeCompositor(Ogre::String compositor)
{
	CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentCompositor, false);
	mCurrentCompositor = compositor;
	CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentCompositor, true);

	if (compositor == "SSAO/CreaseShading")
		showPanel(mCreasePanel);
	 else
		hidePanel(mCreasePanel);
	
	if (compositor == "SSAO/UnsharpMask")
		showPanel(mUnsharpPanel);
	else
		hidePanel(mUnsharpPanel);

	if (compositor == "SSAO/Crytek" || compositor == "SSAO/HorizonBased" || compositor == "SSAO/HemisphereMC" || compositor == "SSAO/Volumetric")
	{
		showWidget(mSamplingCheckbox);
		mSamplingCheckbox->setChecked(mSamplingCheckbox->isChecked()); // easy way to update the sliders...
	}
	else
	{
		hideWidget(mSamplingCheckbox);
		hideWidget(mScreenSpaceSlider);
		hideWidget(mWorldSpaceSlider);
	}

	if (compositor == "SSAO/HemisphereMC")
		showWidget(mSampleLengthExponent);
	else
		hideWidget(mSampleLengthExponent);

	if (compositor == "SSAO/HorizonBased")
		showWidget(mAngleBiasSlider);
	else
		hideWidget(mAngleBiasSlider);


	if (compositor == "SSAO/Crytek")
	{
		showWidget(mOffsetStepSlider);
		showWidget(mEdgeHighlight);
		showWidget(mDefaultAccessibility);
	}
	else
	{
		hideWidget(mOffsetStepSlider);
		hideWidget(mEdgeHighlight);
		hideWidget(mDefaultAccessibility);
	}
}

void Sample_SSAO::buttonHit(OgreBites::Button *button)
{
	if (button->getName() == SSAO_SCREENSHOT_NAME)
	{
		mTrayMgr->hideAll();
		mTakeScreenshot = true;
	}
}

bool Sample_SSAO::frameRenderingQueued(const Ogre::FrameEvent &evt)
{
	if (mTakeScreenshot)
	{
		mWindow->writeContentsToTimestampedFile("screenshot", ".png");
		mTrayMgr->showAll();
		mTakeScreenshot = false;
	}
	
	// call superclass
	return SdkSample::frameRenderingQueued(evt);
}


//-----------------------------------------------------------------------
bool Sample_SSAO::mousePressed( const OIS::MouseEvent& evt, OIS::MouseButtonID id )
{
	if (mTrayMgr->injectMouseDown(evt, id)) 
		return true;
	if (id == OIS::MB_Left) 	
		mTrayMgr->hideCursor();  // hide the cursor if user left-clicks in the scene			

	return true;
}

//-----------------------------------------------------------------------
bool Sample_SSAO::mouseReleased( const OIS::MouseEvent& evt, OIS::MouseButtonID id )
{
	if (mTrayMgr->injectMouseUp(evt, id)) 
		return true;
	if (id == OIS::MB_Left) 
		mTrayMgr->showCursor();  // unhide the cursor if user lets go of LMB

	return true;
}

//-----------------------------------------------------------------------
bool Sample_SSAO::mouseMoved( const OIS::MouseEvent& evt )
{
	// only rotate the camera if cursor is hidden
	if (mTrayMgr->isCursorVisible())
		mTrayMgr->injectMouseMove(evt);
	else 
	{
		mCameraMan->injectMouseMove(evt);
		mCameraMenu->selectItem(SSAO_USER_CAMERA_ITEM);
	}

	return true;
}

void Sample_SSAO::checkBoxToggled(OgreBites::CheckBox *box) 
{
	if (box->getName() == SSAO_SAMPLE_SPACE_NAME)
	{
		setUniform("SSAO/Crytek", "SSAO/Crytek", "cSampleInScreenspace", box->isChecked(), false, 1);
		setUniform("SSAO/HorizonBased", "SSAO/HorizonBased", "cSampleInScreenspace", box->isChecked(), false, 1);
		setUniform("SSAO/HemisphereMC", "SSAO/HemisphereMC", "cSampleInScreenspace", box->isChecked(), false, 1);
		setUniform("SSAO/Volumetric", "SSAO/Volumetric", "cSampleInScreenspace", box->isChecked(), false, 1);
		CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentCompositor, true);

		if (box->isChecked()) // we sample in screen space 
		{
			mTrayMgr->removeWidgetFromTray(mWorldSpaceSlider);
			mWorldSpaceSlider->hide();
			mTrayMgr->moveWidgetToTray(mScreenSpaceSlider, TL_TOPLEFT);
			mScreenSpaceSlider->show();
		}
		else
		{
			mTrayMgr->removeWidgetFromTray(mScreenSpaceSlider);
			mScreenSpaceSlider->hide();
			mTrayMgr->moveWidgetToTray(mWorldSpaceSlider, TL_TOPLEFT);
			mWorldSpaceSlider->show();
		}
	}
}


void Sample_SSAO::setUniform(Ogre::String compositor, Ogre::String material, Ogre::String uniform, float value, bool setVisible, int position)
{
	// remove compositor first???
	CompositorManager::getSingleton().removeCompositor(mViewport, compositor);

	(static_cast<MaterialPtr>(MaterialManager::getSingleton().getByName(material)))->getTechnique(0)->
		getPass(0)->getFragmentProgramParameters()->setNamedConstant(uniform, value);
	
	//adding again
	CompositorManager::getSingleton().addCompositor(mViewport, compositor, position);
	CompositorManager::getSingleton().setCompositorEnabled(mViewport, compositor, setVisible);
}

void Sample_SSAO::changePost(Ogre::String post)
{
	CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentPost, false);
	mCurrentPost = post;
	CompositorManager::getSingleton().setCompositorEnabled(mViewport, mCurrentPost, true);

	if (post == "SSAO/Post/CrossBilateralFilter")
		showPanel(mBilateralPanel);
	else
		hidePanel(mBilateralPanel);
}

void Sample_SSAO::hidePanel(std::vector<Widget *> panel)
{
	for (unsigned int i = 0; i < panel.size(); i++)
	{
		mTrayMgr->removeWidgetFromTray(panel[i]);
		panel[i]->hide();
	}
}

void Sample_SSAO::showPanel(std::vector<Widget *> panel)
{
	for (unsigned int i = 0; i < panel.size(); i++)
	{
		mTrayMgr->moveWidgetToTray(panel[i], TL_TOPLEFT);
		panel[i]->show();
	}
}

void Sample_SSAO::showWidget(Widget* widget)
{
	mTrayMgr->moveWidgetToTray(widget, TL_TOPLEFT);
	widget->show();
}

void Sample_SSAO::hideWidget(Widget* widget)
{
	mTrayMgr->removeWidgetFromTray(widget);
	widget->hide();
}